React = require 'react'
Backbone = require 'backbone'
$ = require 'jquery'
_ = require 'underscore'
Bstr = require 'bumble-strings'

Th = require '../lib/testHelpers'

Select = require('react-select/src/Select')
Select.Async = require('react-select/src/Async')

CollectionPicker = require '../../src/datums/collectionPicker/collectionPicker'

NameCollection = require('../lib/nameCollection')
nameCollection = new NameCollection()

simulateClick = (component) ->
  domNode = Th.domNode(component)
  Th.Simulate.focus(domNode.querySelector('.Select-input input'))
  Th.Simulate.mouseDown(domNode.querySelector('.Select-control'))

testBasicCollectionPickerRender = (component, model, expectedValues, expectedSuggestions) ->
  domNode = Th.domNode(component)
  reactSelect = component.refs.reactSelect
  
  expectedValues ?= nameCollection.get(model.get('nameId')).get('name')
  expectedValues = [expectedValues] unless _.isArray expectedValues
  
  it 'should render a react-select component', ->
    Th.findByClass(component, "Select").length.should.be.equal(1)
    Th.findByTag(component, "input").length.should.be.equal(1)
    expect(reactSelect).to.exist
    
  it 'should show current model value', ->
    expectedNumberOfLabels = expectedValues.length 
    $valueLabels = $(domNode).find('.Select-value-label')
    actualLabelValues = (Bstr.trim($(el).text()) for el in $valueLabels)
    #console.log actualLabelValues
    actualLabelValues.length.should.equal(expectedNumberOfLabels)
    actualLabelValues.should.eql(expectedValues)
  
  
  describe 'when clicked', ->
    before ->
      simulateClick(component)
      #Th.dumpHtml(component)
    
    it 'should initially show all values from collection when clicked', ->
      $selectOptions = $(domNode).find('.Select-option')
      actualOptionValues = (Bstr.trim($(el).text()) for el in $selectOptions)
      
      if component.props.multi
        $selectOptions.length.should.equal(nameCollection.length - expectedValues.length, 
          "as multiselect, expected as many .Select-option elements as collection models minus those selected")
      else    
        $selectOptions.length.should.equal(nameCollection.length, 
          "as single select, expected as many .Select-option elements as collection models")

      if expectedSuggestions?
        actualOptionValues.should.equal expectedSuggestions
      

describe 'CollectionPicker inputMode="edit" as single select', ->
  model = new Backbone.Model({nameId: 11})
  component = Th.render <CollectionPicker attr='nameId' displayAttr="name" model={model} collection={nameCollection} inputMode='edit'/>
  testBasicCollectionPickerRender(component, model)


describe 'CollectionPicker inputMode="edit" as multi select', ->
  model = new Backbone.Model({nameIds: [11, 22]})
  component = Th.render <CollectionPicker multi attr='nameIds' displayAttr="name" model={model} collection={nameCollection} inputMode='edit'/>
  expectedValues = ["Mr. Cuddles", "Sebastian"]
  testBasicCollectionPickerRender(component, model, expectedValues)


# TODO: Find a way to differentiate the rendering of Select vs Select.Async. They seem to render exactly the same in DOM
describe 'CollectionPicker uses Select instead of Select.Async when synchronousLoading=true', ->
  model = new Backbone.Model({nameId: 11})
  component = Th.render <CollectionPicker attr='nameId' displayAttr="name" model={model} synchronousLoading={true} collection={nameCollection} inputMode='edit'/>
  testBasicCollectionPickerRender(component, model)


describe 'CollectionPicker clear value should clear the value and set it to null', ->
  model = new Backbone.Model({nameId: 11})
  component = Th.render <CollectionPicker clearable={true} attr='nameId' displayAttr="name" model={model}
                                          synchronousLoading={true} collection={nameCollection} inputMode='edit'
                                          setOnChange={true} />
  testBasicCollectionPickerRender(component, model)

  domNode = Th.domNode(component)

  describe 'when cleared', ->
    before ->
      Th.Simulate.mouseDown(domNode.querySelector('.Select-clear-zone'))

    it 'should show no label values', ->
      $valueLabels = $(domNode).find('.Select-value-label')
      $valueLabels.length.should.equal(0)
      
    it 'the model should have a null value', ->
      expect(model.get('nameId')).to.equal null


describe 'CollectionPicker as multi select when opened', ->
  model = new Backbone.Model({nameId: 11})
  component = Th.render <CollectionPicker 
    clearable={true} attr='nameId' displayAttr="name"
    collection={nameCollection} inputMode='edit'
    multi searchable
  />

  domNode = Th.domNode(component)

  before ->
    simulateClick(component)  # open the dropdown

  it 'should initially show all values from collection', ->
    $optionElements = $(domNode).find('.Select-option')
    $optionElements.length.should.equal(nameCollection.length)
    

  describe 'and then first option selected', ->
    before ->
      Th.Simulate.mouseDown(domNode.querySelector('.Select-menu .Select-option:first-child'))
      simulateClick(component)  # open the dropdown

    it 'should have selected one option', ->
      # console.log $(domNode).html()
      $optionElements = $(domNode).find('.Select-option.is-focused')
      $optionElements.length.should.equal 1
        
      
      
      
      